@@ -1,6 +1,6 @@
-/* $Id: rmagick.h,v 1.75.2.1 2005/01/11 23:00:56 rmagick Exp $ */
+/* $Id: rmagick.h,v 1.82 2005/04/28 23:41:54 rmagick Exp $ */
 /*=============================================================================
-|               Copyright (C) 2004 by Timothy P. Hunter
+|               Copyright (C) 2005 by Timothy P. Hunter
 | Name:     rmagick.h
 | Purpose:  RMagick declarations and definitions
 | Author:   Tim Hunter
@@ -587,6 +587,9 @@ extern VALUE Info_new(VALUE);
 #endif
 
 extern VALUE Info_define(int, VALUE *, VALUE);
+extern VALUE Info_aset(VALUE, VALUE, VALUE, VALUE);
+extern VALUE Info_aref(VALUE, VALUE, VALUE);
+extern VALUE Info_undefine(VALUE, VALUE, VALUE);
 extern VALUE Info_initialize(VALUE);
 extern VALUE rm_info_new(void);
 
@@ -776,13 +779,16 @@ extern VALUE Image_sample_bang(int, VALUE *, VALUE);
 extern VALUE Image_scale(int, VALUE *, VALUE);
 extern VALUE Image_scale_bang(int, VALUE *, VALUE);
 extern VALUE Image_segment(int, VALUE *, VALUE);
+extern VALUE Image_sepiatone(int, VALUE *, VALUE);
 extern VALUE Image_set_channel_depth(VALUE, VALUE, VALUE);
 extern VALUE Image_shade(int, VALUE *, VALUE);
+extern VALUE Image_shadow(int, VALUE *, VALUE);
 extern VALUE Image_sharpen(int, VALUE *, VALUE);
 extern VALUE Image_sharpen_channel(int, VALUE *, VALUE);
 extern VALUE Image_shave(VALUE, VALUE, VALUE);
 extern VALUE Image_shave_bang(VALUE, VALUE, VALUE);
 extern VALUE Image_shear(VALUE, VALUE, VALUE);
+extern VALUE Image_sigmoidal_contrast_channel(int, VALUE *, VALUE);
 extern VALUE Image_signature(VALUE);
 extern VALUE Image_solarize(int, VALUE *, VALUE);
 extern VALUE Image_spaceship(VALUE, VALUE);
@@ -930,8 +936,8 @@ extern char  *rm_string_value_ptr(volatile VALUE *);
 extern char  *rm_string_value_ptr_len(volatile VALUE *, long *);
 extern void   rm_check_ary_len(VALUE, int);
 extern void   rm_check_frozen(VALUE);
-extern VALUE  rm_obj_to_s(VALUE);
-extern double rm_fuzz_to_dbl(VALUE fuzz);
+extern double rm_fuzz_to_dbl(VALUE);
+extern double rm_percentage(VALUE);
 extern VALUE  rm_define_enum_type(char *);
 extern void   rm_write_temp_image(Image *, char *);
 extern void   rm_delete_temp_image(char *);
