@@ -1,6 +1,6 @@
-/* $Id: rminfo.c,v 1.17 2004/02/14 19:35:47 rmagick Exp $ */
+/* $Id: rminfo.c,v 1.28 2005/04/30 23:01:13 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2004 by Timothy P. Hunter
+|                Copyright (C) 2005 by Timothy P. Hunter
 | Name:     rminfo.c
 | Author:   Tim Hunter
 | Purpose:  Info class method definitions for RMagick.
@@ -10,6 +10,148 @@
 
 DEF_ATTR_ACCESSOR(Info, antialias, bool)
 
+/*
+ * Method:  value = Info[format, key]
+ * Purpose: get the value of an option set by Info[]=
+*/
+#define MAX_FORMAT_LEN 60
+
+VALUE
+Info_aref(VALUE self, VALUE format, VALUE key)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    char *format_p, *key_p;
+    long format_l, key_l;
+    const char *value;
+    char fkey[MaxTextExtent];
+
+    format_p = STRING_PTR_LEN(format, format_l);
+    key_p = STRING_PTR_LEN(key, key_l);
+
+    if (format_l > MAX_FORMAT_LEN || format_l + key_l > MaxTextExtent-1)
+    {
+        rb_raise(rb_eArgError, "can't reference %.60s:%.1024s - too long", format_p, key_p);
+    }
+
+    sprintf(fkey, "%.60s:%.*s", STRING_PTR(format), (int)(MaxTextExtent-61), STRING_PTR(key));
+
+    Data_Get_Struct(self, Info, info);
+    value = GetImageOption(info, fkey);
+    if (!value)
+    {
+        return Qnil;
+    }
+
+    return rb_str_new2(value);
+
+#elif defined(HAVE_ADDDEFINITIONS)
+    Info *info;
+    const char *value;
+
+    Data_Get_Struct(self, Info, info);
+    value = AccessDefinition(info, STRING_PTR(format), STRING_PTR(key));
+
+    if (!value)
+    {
+        return Qnil;
+    }
+
+    return rb_str_new2(value);
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
+/*
+    Method:     Info[format, key] = value
+    Purpose:    Call AddDefinitions (GM) or SetImageOption (IM)
+    Note:       Essentially the same function as Info#define but paired
+                with Info#[]=
+*/
+VALUE
+Info_aset(VALUE self, VALUE format, VALUE key, VALUE value)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    char *format_p, *key_p, *value_p = "";
+    long format_l, key_l;
+    char ckey[MaxTextExtent];
+    unsigned int okay;
+
+
+    Data_Get_Struct(self, Info, info);
+
+    format_p = STRING_PTR_LEN(format, format_l);
+    key_p = STRING_PTR_LEN(key, key_l);
+
+    /* Allow any argument that supports to_s */
+    value = rb_funcall(value, ID_to_s, 0);
+    value_p = STRING_PTR(value);
+
+    if (format_l > MAX_FORMAT_LEN || format_l+key_l > MaxTextExtent-1)
+    {
+        rb_raise(rb_eArgError, "%.60s:%.1024s not defined - too long", format_p, key_p);
+    }
+
+    (void) sprintf(ckey, "%.60s:%.*s", format_p, (int)(sizeof(ckey)-MAX_FORMAT_LEN), key_p);
+
+    okay = SetImageOption(info, ckey, value_p);
+    if (!okay)
+    {
+        rb_warn("%.60s:%.1024s not defined - SetImageOption failed.", format_p, key_p);
+        return Qnil;
+    }
+
+    return self;
+
+#elif defined(HAVE_ADDDEFINITIONS)
+    Info *info;
+    char *format_p, *key_p, *value_p = NULL;
+    long format_l, key_l, value_l = 0;
+    unsigned int okay;
+    ExceptionInfo exception;
+    char definitions[MaxTextExtent*2];/* Make this buffer longer than the buffer used   */
+                                      /* for SetImageOptions since AddDefinitions cats  */
+                                      /* the value onto the format:key pair.            */
+
+    Data_Get_Struct(self, Info, info);
+
+    format_p = STRING_PTR_LEN(format, format_l);
+    key_p = STRING_PTR_LEN(key, key_l);
+    value = rb_funcall(value, ID_to_s, 0);
+    value_p = STRING_PTR_LEN(value, value_l);
+
+    if ((3 + format_l + key_l + value_l) > sizeof(definitions))
+    {
+        rb_raise(rb_eArgError, "%.60s:%.1024s not defined - too long", format_p, key_p);
+    }
+    (void)sprintf(definitions, "%s:%s=", format_p, key_p);
+    if (value_l > 0)
+    {
+        strcat(definitions, value_p);
+    }
+
+    GetExceptionInfo(&exception);
+    okay = AddDefinitions(info, definitions, &exception);
+    HANDLE_ERROR
+    if (!okay)
+    {
+        rb_warn("%.60s:%.1024s not defined - AddDefinitions failed.", format_p, key_p);
+        return Qnil;
+    }
+
+    return self;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
+
 /*
     Method:     Info#background_color
     Purpose:    return the name of the background color as a String
@@ -148,7 +290,7 @@ Info_define(int argc, VALUE *argv, VALUE self)
     {
         case 3:
             /* Allow any argument that supports to_s */
-            fmt_arg = rb_funcall(argv[2], to_s_ID, 0);
+            fmt_arg = rb_funcall(argv[2], ID_to_s, 0);
             value = STRING_PTR(fmt_arg);
         case 2:
             key = STRING_PTR_LEN(argv[1], key_l);
@@ -190,7 +332,7 @@ Info_define(int argc, VALUE *argv, VALUE self)
     {
         case 3:
             /* Allow any argument that supports to_s */
-            fmt_arg = rb_funcall(argv[2], to_s_ID, 0);
+            fmt_arg = rb_funcall(argv[2], ID_to_s, 0);
             value = STRING_PTR_LEN(fmt_arg, value_l);
             /* Fall through */
         case 2:
@@ -225,11 +367,12 @@ Info_define(int argc, VALUE *argv, VALUE self)
     return self;
 
 #else
-    not_implemented("define");
+    rm_not_implemented();
     return (VALUE)0;
 #endif
 }
 
+
 DEF_ATTR_READER(Info, density, str)
 
 /*
@@ -253,7 +396,7 @@ Info_density_eq(VALUE self, VALUE density_arg)
         return self;
     }
 
-    density = rb_funcall(density_arg, to_s_ID, 0);
+    density = rb_funcall(density_arg, ID_to_s, 0);
     dens = STRING_PTR(density);
     if (!IsGeometry(dens))
     {
@@ -328,7 +471,7 @@ Info_extract_eq(VALUE self, VALUE extract_arg)
         return self;
     }
 
-    extract = rb_funcall(extract_arg, to_s_ID, 0);
+    extract = rb_funcall(extract_arg, ID_to_s, 0);
     extr = STRING_PTR(extract);
     if (!IsGeometry(extr))
     {
@@ -374,13 +517,13 @@ Info_tile_eq(VALUE self, VALUE tile)
 VALUE
 Info_extract(VALUE self)
 {
-    not_implemented("extract");
+    rm_not_implemented();
     return (VALUE)0;
 }
 VALUE
 Info_extract_eq(VALUE self, VALUE extr)
 {
-    not_implemented("extract=");
+    rm_not_implemented();
     return (VALUE)0;
 }
 
@@ -410,7 +553,7 @@ Info_tile_eq(VALUE self, VALUE tile_arg)
         return self;
     }
 
-    tile = rb_funcall(tile_arg, to_s_ID, 0);
+    tile = rb_funcall(tile_arg, ID_to_s, 0);
     til = STRING_PTR(tile);
     if (!IsGeometry(til))
     {
@@ -543,45 +686,18 @@ DEF_ATTR_READER(Info, fuzz, dbl)
 /*
     Method:     Info#fuzz=number
                 Info#fuzz=NN%
-    Notes:      If the fuzz value is a string in the form 'NN%',
-                set the fuzz to a percentage of MaxRGB.
-                See Image#fuzz
+    Notes:      See Image#fuzz
 */
-VALUE Info_fuzz_eq(VALUE self, VALUE fuzz_arg)
+VALUE Info_fuzz_eq(VALUE self, VALUE fuzz)
 {
     Info *info;
-    char *fuzz_str, *end;
-    double fuzz;
-
-    // If the fuzz argument is a string, convert it to a double.
-    // If the argument ends in '%', treat it as a percentage.
-    if(TYPE(fuzz_arg) == T_STRING)
-    {
-        fuzz_str = STRING_PTR(fuzz_arg);
-        fuzz = strtod(fuzz_str, &end);
-        if(*end == '%')
-        {
-            fuzz = (fuzz * MaxRGB) / 100;
-        }
-        else if(*end != '\0')
-        {
-            rb_raise(rb_eTypeError, "expected percentage, got `%s'", fuzz_str);
-        }
-    }
-
-    // Otherwise it's supposed to be a number.
-    else
-    {
-        fuzz = NUM2DBL(fuzz_arg);
-    }
 
     Data_Get_Struct(self, Info, info);
-
-    info->fuzz = fuzz;
-
+    info->fuzz = rm_fuzz_to_dbl(fuzz);
     return self;
 }
 
+
 DEF_ATTR_ACCESSOR(Info, group, long)
 
 /*
@@ -715,7 +831,7 @@ Info_page_eq(VALUE self, VALUE page_arg)
         info->page = NULL;
         return self;
     }
-    geom_str = rb_funcall(page_arg, to_s_ID, 0);
+    geom_str = rb_funcall(page_arg, ID_to_s, 0);
     geometry=PostscriptGeometry(STRING_PTR(geom_str));
     if (*geometry == '\0')
     {
@@ -861,7 +977,7 @@ Info_size_eq(VALUE self, VALUE size_arg)
         return self;
     }
 
-    size = rb_funcall(size_arg, to_s_ID, 0);
+    size = rb_funcall(size_arg, ID_to_s, 0);
     sz = STRING_PTR(size);
     if (!IsGeometry(sz))
     {
@@ -873,6 +989,68 @@ Info_size_eq(VALUE self, VALUE size_arg)
     return self;
 }
 
+
+/*
+    Method:     Info#undefine
+    Purpose:    Undefine image option
+*/
+
+VALUE
+Info_undefine(VALUE self, VALUE format, VALUE key)
+{
+#if defined(HAVE_SETIMAGEOPTION)
+    Info *info;
+    char *format_p, *key_p;
+    long format_l, key_l;
+    char fkey[MaxTextExtent];
+
+    format_p = STRING_PTR_LEN(format, format_l);
+    key_p = STRING_PTR_LEN(key, key_l);
+
+    if (format_l > MAX_FORMAT_LEN || format_l + key_l > MaxTextExtent)
+    {
+        rb_raise(rb_eArgError, "can't undefine %.60s:%.1024s - too long", format_p, key_p);
+    }
+
+    sprintf(fkey, "%.60s:%.*s", format_p, (int)(MaxTextExtent-61), key_p);
+
+    Data_Get_Struct(self, Info, info);
+    /* Depending on the IM version, RemoveImageOption returns either */
+    /* char * or MagickBooleanType. Ignore the return value.         */
+    (void) RemoveImageOption(info, fkey);
+    return self;
+
+#elif defined(HAVE_ADDDEFINITIONS)
+    Info *info;
+    unsigned int okay;
+    char *format_p, *key_p;
+    long format_l, key_l;
+    char fkey[MaxTextExtent];
+
+    format_p = STRING_PTR_LEN(format, format_l);
+    key_p = STRING_PTR_LEN(key, key_l);
+
+    if (format_l > MAX_FORMAT_LEN || format_l + key_l > MaxTextExtent)
+    {
+        rb_raise(rb_eArgError, "can't undefine %.60s:%.1024s - too long", format_p, key_p);
+    }
+
+    sprintf(fkey, "%.60s:%.*s", format_p, MaxTextExtent-61, key_p);
+
+    Data_Get_Struct(self, Info, info);
+    okay = RemoveDefinitions(info, fkey);
+    if (!okay)
+    {
+        rb_raise(rb_eArgError, "no such key: `%s'", fkey);
+    }
+    return self;
+
+#else
+    rm_not_implemented();
+    return (VALUE)0;
+#endif
+}
+
 /*
     Method:  Info#units
     Purpose: Get the resolution type
