@@ -1,6 +1,6 @@
-/* $Id: rmmain.c,v 1.74.2.2 2005/02/23 15:09:50 rmagick Exp $ */
+/* $Id: rmmain.c,v 1.84 2005/04/30 23:01:39 rmagick Exp $ */
 /*============================================================================\
-|                Copyright (C) 2004 by Timothy P. Hunter
+|                Copyright (C) 2005 by Timothy P. Hunter
 | Name:     rmmain.c
 | Author:   Tim Hunter
 | Purpose:  Contains all module, class, method declarations.
@@ -602,7 +602,9 @@ Init_RMagick(void)
     DCL_ATTR_WRITER(Image, opacity)
     DCL_ATTR_READER(Image, orientation)
     DCL_ATTR_ACCESSOR(Image, page)
+#if defined(HAVE_IMAGE_QUALITY)
     DCL_ATTR_READER(Image, quality)
+#endif
     DCL_ATTR_READER(Image, quantum_depth)
     DCL_ATTR_ACCESSOR(Image, rendering_intent)
     DCL_ATTR_READER(Image, rows)
@@ -729,13 +731,16 @@ Init_RMagick(void)
     rb_define_method(Class_Image, "scale", Image_scale, -1);
     rb_define_method(Class_Image, "scale!", Image_scale_bang, -1);
     rb_define_method(Class_Image, "segment", Image_segment, -1);
+    rb_define_method(Class_Image, "sepiatone", Image_sepiatone, -1);
     rb_define_method(Class_Image, "set_channel_depth", Image_set_channel_depth, 2);
     rb_define_method(Class_Image, "shade", Image_shade, -1);
+    rb_define_method(Class_Image, "shadow", Image_shadow, -1);
     rb_define_method(Class_Image, "sharpen", Image_sharpen, -1);
     rb_define_method(Class_Image, "sharpen_channel", Image_sharpen_channel, -1);
     rb_define_method(Class_Image, "shave", Image_shave, 2);
     rb_define_method(Class_Image, "shave!", Image_shave_bang, 2);
     rb_define_method(Class_Image, "shear", Image_shear, 2);
+    rb_define_method(Class_Image, "sigmoidal_contrast_channel", Image_sigmoidal_contrast_channel, -1);
     rb_define_method(Class_Image, "signature", Image_signature, 0);
     rb_define_method(Class_Image, "solarize", Image_solarize, -1);
     rb_define_method(Class_Image, "<=>", Image_spaceship, 1);
@@ -923,6 +928,9 @@ Init_RMagick(void)
     rb_define_method(Class_Info, "initialize", Info_initialize, 0);
     rb_define_method(Class_Info, "freeze", rm_no_freeze, 0);
     rb_define_method(Class_Info, "define", Info_define, -1);
+    rb_define_method(Class_Info, "[]=", Info_aset, 3);
+    rb_define_method(Class_Info, "[]", Info_aref, 2);
+    rb_define_method(Class_Info, "undefine", Info_undefine, 2);
 
     DCL_ATTR_ACCESSOR(Info, antialias)
     DCL_ATTR_ACCESSOR(Info, background_color)
@@ -1185,6 +1193,9 @@ Init_RMagick(void)
         ENUMERATOR(FaxCompression)
         ENUMERATOR(Group4Compression)
         ENUMERATOR(JPEGCompression)
+#if defined(HAVE_JPEG2000COMPRESSION)
+        ENUMERATOR(JPEG2000Compression)
+#endif
         ENUMERATOR(LosslessJPEGCompression)
         ENUMERATOR(LZWCompression)
         ENUMERATOR(RunlengthEncodedCompression)
@@ -1439,7 +1450,7 @@ Init_RMagick(void)
 
 
     // Magick::Primary
-    Class_Primary = rb_struct_define(NULL, "x", "y", "z", 0);
+    Class_Primary = rb_struct_define(NULL, "x", "y", "z", NULL);
     rb_define_method(Class_Primary, "to_s", PrimaryInfo_to_s, 0);
     rb_define_const(Module_Magick, "Primary", Class_Primary);
 
@@ -1496,14 +1507,14 @@ Init_RMagick(void)
 static void version_constants(void)
 {
     const char *mgk_version;
-    char long_version[300];
+    char long_version[400];
 
     mgk_version = GetMagickVersion(NULL);
     rb_define_const(Module_Magick, "Magick_version", rb_str_new2(mgk_version));
 
     rb_define_const(Module_Magick, "Version", rb_str_new2(PACKAGE_STRING));
     sprintf(long_version,
-        "This is %s ($Date: 2005/02/23 15:09:50 $) Copyright (C) 2004 by Timothy P. Hunter\n"
+        "This is %s ($Date: 2005/04/30 23:01:39 $) Copyright (C) 2005 by Timothy P. Hunter\n"
         "Built with %s\n"
         "Built for %s\n"
         "Web page: http://rmagick.rubyforge.org\n"
